@@ -41,7 +41,7 @@ import os
 import re
 from collections import namedtuple
 import datetime
-import urllib
+from contextlib import contextmanager
 
 import numpy as np
 import six
@@ -49,57 +49,6 @@ import jpype
 
 import cmmnbuild_dep_manager
 
-jarname = re.compile(r"([a-z\-]+)-([0-9.]+)\.jar")
-
-
-def ver2num(ver):
-    out = 0
-    for ii, vv in enumerate(map(int, reversed(ver.split(".")))):
-        out += vv * 1000 ** ii
-    return out
-
-
-def get_jarversion(jars):
-    out = {}
-    for jar in jars:
-        res = jarname.search(jar)
-        if res:
-            name, version = res.groups()
-            out[name] = version
-    return out
-
-
-def older_jar_than_pro(jars):
-    jar_mgr = get_jarversion(jars)
-    # del jar_mgr['cmw-directory-client']
-    url = "http://bewww.cern.ch/ap/deployments/applications/cern/lsa/lsa-app-suite/PRO/lsa-app-suite.jnlp"
-    jar_pro = get_jarversion(urllib.urlopen(url).readlines())
-    result = False
-    for jar in set(jar_mgr).intersection(jar_pro):
-        v1 = jar_mgr[jar]
-        v2 = jar_pro[jar]
-        vv1 = ver2num(jar_mgr[jar])
-        vv2 = ver2num(jar_pro[jar])
-        if vv1 < vv2:
-            print("Checking %-30s: USED=%-6s < PRO=%-6s" % (jar, v1, v2))
-            result = True
-    return result
-
-
-def check_lsa():
-    mgrjars = cmmnbuild_dep_manager.Manager().jars()
-    lsajars = [j for j in mgrjars if "lsa" in j]
-    if len(lsajars) == 0:
-        raise ImportError("LSA jars not (yet) installed")
-
-
-#    elif older_jar_than_pro(mgrjars):
-#           print("Installed jar older than PRO version. Please rerun 'python -m cmmnbuild_dep_manager resolve'")
-#           raise ImportError("Installed jar older than PRO version.")
-
-# check_lsa()
-
-
 # Python data descriptors
 TrimHeader = namedtuple(
     "TrimHeader", ["id", "beamProcesses", "createdDate", "description", "clientInfo"],
@@ -125,9 +74,6 @@ PCInfo = namedtuple(
 Context = namedtuple("Context", ["timestamp", "name", "user"])
 
 
-#
-
-
 def _build_TrimHeader(th):
     return TrimHeader(
         id=th.getId(),
@@ -174,11 +120,8 @@ class LSAClient(object):
         self._java = jpype.JPackage("java")
         self._System = self._java.lang.System
 
-        # no java operation at import time because java might not be available and
-        #  mgr.resolve will (silently) fail
-        if callable(self._org.apache.log4j.varia.NullAppender):
-            null = self._org.apache.log4j.varia.NullAppender()
-            self._org.apache.log4j.BasicConfigurator.configure(null)
+        null = self._org.apache.log4j.varia.NullAppender()
+        self._org.apache.log4j.BasicConfigurator.configure(null)
 
         # System.setProperty("lsa.mode", "3")
 
@@ -249,6 +192,23 @@ class LSAClient(object):
         self._deviceService = self._ServiceLocator.getService(self._DeviceService)
         self._fidelService = self._ServiceLocator.getService(self._FidelService)
 
+    @contextmanager
+    def java_api(self):
+        """Run code with the access to the LSA Java universe through JPype imports.
+        After the import/execution returns, the original python import behavior is restored.
+
+        This method is typically called from a launcher module, e.g.:
+            `with pjlsa.LSAClient(server=...).java_api(): import my_main_module`
+        """
+        with self._mgr.imports():
+            # work-around to fire up JPypes forward converters - TODO: remove me in JPype 0.8!
+            from java.util import HashSet, HashMap, ArrayList, Date
+            ArrayList()
+            HashMap()
+            HashSet()
+            Date()
+            yield
+
     def _getContextFamily(self, name):
         if isinstance(name, str):
             return getattr(self._ContextFamily, name.upper())
@@ -457,7 +417,7 @@ class LSAClient(object):
         return [
             _build_TrimHeader(th)
             for th in self._getRawTrimHeadersByBeamprocess(
-                    self._buildParameterList(parameter), beamprocess, start, end
+                self._buildParameterList(parameter), beamprocess, start, end
             )
         ]
 
@@ -468,9 +428,9 @@ class LSAClient(object):
                 self._buildParameterList(parameter), cycle, start, end
             )
         ]
-        
+
     def getTrimHeaders(
-        self, parameter, beamprocess=None, cycle=None, start=None, end=None
+            self, parameter, beamprocess=None, cycle=None, start=None, end=None
     ):
         if beamprocess is not None:
             return self._getTrimHeadersByBeamprocess(parameter,
@@ -482,7 +442,7 @@ class LSAClient(object):
                                                start=start, end=end)
 
     def _getTrimsByBeamprocess(
-        self, parameter, beamprocess, start=None, end=None, part="value"
+            self, parameter, beamprocess, start=None, end=None, part="value"
     ):
         parameterList = self._buildParameterList(parameter)
         bp = self._getBeamProcess(beamprocess)
@@ -538,7 +498,7 @@ class LSAClient(object):
         return out
 
     def _getTrimsByCycle(
-        self, parameter, cycle, start=None, end=None, part="value"
+            self, parameter, cycle, start=None, end=None, part="value"
     ):
         parameterList = self._buildParameterList(parameter)
         cy = self._getCycle(cycle)
@@ -546,7 +506,7 @@ class LSAClient(object):
         timestamps = {}
         values = {}
         for th in self._getRawTrimHeadersByCycle(
-            parameterList, cy, start, end
+                parameterList, cy, start, end
         ):
             csrb = self._domain.settings.ContextSettingsRequestBuilder()
             csrb.standAloneContext(cy)
@@ -596,8 +556,9 @@ class LSAClient(object):
         for name in values:
             out[name] = TrimTuple(time=timestamps[name], data=values[name])
         return out
+
     def getTrims(
-        self, parameter, beamprocess=None, cycle=None, start=None, end=None, part="value"
+            self, parameter, beamprocess=None, cycle=None, start=None, end=None, part="value"
     ):
         if beamprocess is not None:
             return self._getTrimsByBeamprocess(parameter,
@@ -623,17 +584,17 @@ class LSAClient(object):
             parameter, cycle, part=part, start=th.createdDate
         )[parameter]
         return TrimTuple(res.time[-1], res.data[-1])
-    
+
     def getLastTrim(
-        self, parameter, beamprocess=None, cycle=None, part="value"
+            self, parameter, beamprocess=None, cycle=None, part="value"
     ):
         if beamprocess is not None:
             return self._getLastTrimByBeamprocess(parameter,
-                                                  beamprocess=beamprocess, 
+                                                  beamprocess=beamprocess,
                                                   part=part)
         else:
             return self._getLastTrimByCycle(parameter,
-                                            cycle=cycle, 
+                                            cycle=cycle,
                                             part=part)
 
     def _getLastTrimValueByBeamprocess(self, parameter, beamprocess, part="value"):
@@ -649,19 +610,19 @@ class LSAClient(object):
             parameter, cycle, part=part, start=th.createdDate
         )[parameter]
         return res.data[-1]
-    
+
     def getLastTrimValue(
-        self, parameter, beamprocess=None, cycle=None, part="value"
+            self, parameter, beamprocess=None, cycle=None, part="value"
     ):
         if beamprocess is not None:
             return self._getLastTrimValueByBeamprocess(parameter,
-                                                       beamprocess=beamprocess, 
+                                                       beamprocess=beamprocess,
                                                        part=part)
         else:
             return self._getLastTrimValueByCycle(parameter,
-                                                 cycle=cycle, 
+                                                 cycle=cycle,
                                                  part=part)
-            
+
     def getOpticTable(self, beamprocess):
         bp = self._getBeamProcess(beamprocess)
         if bp is None:
@@ -734,6 +695,16 @@ class LSAClient(object):
             pcname = list(pcs[pcname])[0].toString()
         return pcname
 
+    def findMadStrengthNameByPCName(self, pcname, full=False):
+        if full is True:
+            nl = self._java.util.Collections.singleton(pcname)
+            pcs = self._deviceService.findLogicalHardwaresByActualDeviceNames(nl)
+            pcname = list(pcs[pcname])[0].toString()
+        nl = self._java.util.Collections.singleton(pcname)
+        madnames = self._deviceService.findMadStrengthNamesByLogicalNames(nl)
+        madname = madnames[pcname]
+        return madname
+
     def getCalibration(self, pcname, fieldtype="B_FIELD"):
         jfieldtype = getattr(self._CalibrationFunctionTypes, fieldtype)
         cal = self._fidelService.findCalibrationByLogicalHardware(pcname)
